//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "DetectorConstruction.hh"
#include "G4RunManager.hh"

#include "G4Material.hh"
#include "G4NistManager.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Ellipsoid.hh"

#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4SystemOfUnits.hh"
#include "G4PhysicalConstants.hh"

#include "G4RotationMatrix.hh"
#include "G4ThreeVector.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4MultiFunctionalDetector.hh"
#include "G4PSEnergyDeposit.hh"
#include "G4SDManager.hh"
#include "G4SDParticleFilter.hh"
#include "G4SDKineticEnergyFilter.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::DetectorConstruction(){}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

DetectorConstruction::~DetectorConstruction(){}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void DetectorConstruction::DefineMaterials()
{ 
	//variables
	G4String name, symbol;
	G4int Z;
	G4double A;	
	G4double density = 1.29*mg/cm3;
	G4int nc = 2;
	
	//air
	G4Element* elN = new G4Element(name="Nitrogen", symbol="N", Z=7, A=14.01*g/mole);
	G4Element* elO = new G4Element("Oxygen", "O", 8, 16.*g/mole);
	air = new G4Material("air", density, nc);
	air->AddElement(elN,0.7);
	air->AddElement(elO,0.3);
	
	//water
	Z = 1;
	A = 1.01*g/mole;
    density = 1.*g/cm3;
	G4Element* elH = new G4Element("Hydrogen", "H", Z, A);
	water = new G4Material("water", density, nc);
	water->AddElement(elH,2);
	water->AddElement(elO,1);
	
	//Define materials from the NIST database
	//Lead = G4NistManager::Instance()->FindOrBuildMaterial("G4_Pb");
  	G4NistManager* NistMan = G4NistManager::Instance();
  	Lead = NistMan->FindOrBuildMaterial("G4_Pb"); 

    //Print all the materials defined
  	G4cout << *(G4Material::GetMaterialTable()) << G4endl;
    G4cout << "###############################" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* DetectorConstruction::Construct()
{
	//recall defined materials    
    DefineMaterials(); 

    //World
    G4Material* Galactic = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
    
    //G4double worldSize = 1. * CLHEP::meter;
    G4double worldSize = 1. * m;
    
    G4Box* worldSolid = new G4Box("World",
                                  worldSize/2.,
                                  worldSize/2.,
                                  worldSize/2.);
    
    G4LogicalVolume* worldLogic = new G4LogicalVolume(worldSolid,
                                                      Galactic,
                                                      "World");
    
    G4PVPlacement* worldPhysical = new G4PVPlacement(0,
                                                     G4ThreeVector(),
                                                     worldLogic,
                                                     "World",
                                                     0,
                                                     false,
                                                     0);
                                                     
    
    //variable for enabling the geometry check
    G4bool check_overlap = true;
        
    //definition of a solid
   	G4double bx = 10.*cm;
   	G4double by = 20.*cm;
   	G4double bz = 30.*cm;    
   	G4Box* myBox = new G4Box("myBox",
                             bx/2.,
                             by/2.,
                             bz/2.);
    
    //logical Volume
    myLogicalVolume = new G4LogicalVolume(myBox,
                                          water,
                                          "myLogicalVolume");
    
    //visualization attributes
    G4VisAttributes* myBoxVisAttribute = new G4VisAttributes(G4Colour(1., 1., 0.));  
	myBoxVisAttribute->SetForceSolid(true);
	myLogicalVolume->SetVisAttributes(myBoxVisAttribute);
                                                   
    //define a rotation matrix
	G4RotationMatrix* myRotationMatrix = new G4RotationMatrix();
	myRotationMatrix->rotateY(90.*CLHEP::deg);
	myRotationMatrix->rotateZ(0.*CLHEP::deg);
	
	//define a translation vector
	G4double sx = 0.*cm;
   	G4double sy = 0.*cm;
   	G4double sz = 10.*cm;     	
   	G4ThreeVector myTranslationVector = G4ThreeVector(sx, sy, sz);
    
    //placement of the solid
    myPhysicalVolume = new G4PVPlacement(myRotationMatrix,
                                         myTranslationVector,
                                         myLogicalVolume,
                                         "myPhysicalVolume",
                                         worldLogic,
                                         false,
                                         check_overlap);
                                         
      
      
    G4double shape1_sizexy = 10.*mm;
    G4double shape1_sizez = 10.*mm;
                                                        
    G4Ellipsoid* solidShape1_3 = new G4Ellipsoid("Shape1_3", 
                                                 shape1_sizexy/2, 
                                                 shape1_sizexy/2, 
                                                 shape1_sizez, 
                                                 0, 144.5*mm);
    G4Tubs* solidShape1_4 = new G4Tubs("Shape1_4", 0, 160/2*mm, 10*mm, 0, 2*CLHEP::pi);
  /*  
    G4MultiUnion* solidShape1 = new G4MultiUnion("Shape1");    
    G4RotationMatrix rotm  = G4RotationMatrix();
    G4ThreeVector position3 = G4ThreeVector(0*mm, 0*mm, 75*mm);
    G4ThreeVector position4 = G4ThreeVector(0*mm, 0*mm, (75+144.5+10)*mm);
    G4Transform3D tr3 = G4Transform3D(rotm, position3);
    G4Transform3D tr4 = G4Transform3D(rotm, position4);
    solidShape1->AddNode(*solidShape1_3, tr3);
    solidShape1->AddNode(*solidShape1_4, tr4);
    solidShape1->Voxelize();
  */  
    
    G4LogicalVolume* myEllissoidLV = new G4LogicalVolume(solidShape1_4,
    													  water,
    													  "myEllissoidLV");
    													
 	G4VPhysicalVolume* myEllissoidPV = new G4PVPlacement(0,
  												     G4ThreeVector(-30.*cm, -30.*cm, -30.*cm),	
                                                     myEllissoidLV, //pointer of logical
                                                     "myEllissoidPV", //name
                                                     worldLogic, //pointer of mother
                                                     false, //not important
                                                     0, //copy number
                                                     check_overlap); //geomtery check   
 

    return worldPhysical;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void DetectorConstruction::ConstructSDandField()
{
    //Primitive scorer
    G4MultiFunctionalDetector* multisd = new G4MultiFunctionalDetector("multisd");
    G4VPrimitiveScorer* EdepDet = new G4PSEnergyDeposit("edep");
    multisd->RegisterPrimitive(EdepDet);
    G4SDManager::GetSDMpointer()->AddNewDetector(multisd);
	
	//SetSensitiveDetector("myLogicalVolume",multisd);
	SetSensitiveDetector(myLogicalVolume->GetName(),multisd);
	//myLogicalVolume->SetSensitiveDetector(multisd);
	
	G4SDParticleFilter* particleFilter = new G4SDParticleFilter("particleFilter");
	particleFilter->add("e-");  
	EdepDet->SetFilter(particleFilter);      
/*	
	G4SDKineticEnergyFilter* energyFilter = new G4SDKineticEnergyFilter("EnergyFilter"); 
    energyFilter->SetKineticEnergy(20.*keV, 50.*keV);
    EdepDet->SetFilter(energyFilter);  
*/
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo..
